class MigrateToRbacSystem < ActiveRecord::Migration[8.0]
  def up
    # Create roles
    shift_coordinator_role = Role.create!(name: 'shift_coordinator', description: 'Can manage session assignments and scheduling')
    events_admin_role = Role.create!(name: 'events_admin', description: 'Can manage conferences and all sub-resources')

    # Create permissions
    manage_assignments = Permission.create!(name: 'manage_assignments', description: 'Can create and delete assignments')
    manage_conferences = Permission.create!(name: 'manage_conferences', description: 'Can create, edit, and delete conferences')
    manage_sessions = Permission.create!(name: 'manage_sessions', description: 'Can create, edit, and delete sessions')
    manage_speakers = Permission.create!(name: 'manage_speakers', description: 'Can create, edit, and delete speakers')
    manage_stages = Permission.create!(name: 'manage_stages', description: 'Can create, edit, and delete stages')

    # Associate permissions with roles
    shift_coordinator_role.permissions << manage_assignments

    events_admin_role.permissions << manage_conferences
    events_admin_role.permissions << manage_sessions
    events_admin_role.permissions << manage_speakers
    events_admin_role.permissions << manage_stages

    # Migrate existing shift coordinators to the new role system
    User.where(shiftcoordinator: true).find_each do |user|
      user.roles << shift_coordinator_role
      puts "Migrated user #{user.name} to shift_coordinator role"
    end

    # Add a column to track migration completion
    add_column :users, :migrated_to_rbac, :boolean, default: false

    # Mark all users as migrated
    User.update_all(migrated_to_rbac: true)

    # Only remove the shiftcoordinator column after we are sure all users are migrated
    # We'll do this in a separate migration after verifying
  end

  def down
    # Restore shift coordinator status
    if column_exists?(:users, :shiftcoordinator) && column_exists?(:users, :migrated_to_rbac)
      User.where(migrated_to_rbac: true).find_each do |user|
        user.update(shiftcoordinator: user.has_role?('shift_coordinator'))
      end
    end

    # Remove the migration tracking column if it exists
    remove_column :users, :migrated_to_rbac if column_exists?(:users, :migrated_to_rbac)

    # No need to delete roles and permissions as they will be dropped with their tables
  end
end
