namespace :admin do
  desc "Assign admin role to a user by email"
  task :make_admin, [ :email ] => :environment do |t, args|
    email = args[:email]

    if email.blank?
      puts "Error: Email is required."
      puts "Usage: rake admin:make_admin[user@example.com]"
      exit 1
    end

    user = User.find_by(email: email)

    if user.nil?
      puts "Error: User with email '#{email}' not found."
      exit 1
    end

    # Get the shift_coordinator role
    admin_role = Role.find_by(name: "shift_coordinator")

    if admin_role.nil?
      puts "Error: 'shift_coordinator' role does not exist."
      exit 1
    end

    if user.has_role?("shift_coordinator")
      puts "User '#{user.name}' already has the admin role."
    else
      user.roles << admin_role
      puts "Successfully assigned admin role to '#{user.name}' (#{user.email})."
    end
  end

  desc "List all admin users"
  task list: :environment do
    admin_role = Role.find_by(name: "shift_coordinator")

    if admin_role.nil?
      puts "Error: 'shift_coordinator' role does not exist."
      exit 1
    end

    admins = admin_role.users

    if admins.empty?
      puts "No users with admin rights found."
    else
      puts "Users with admin rights:"
      puts "-----------------------"
      admins.each do |admin|
        puts "#{admin.name} (#{admin.email || 'No email'})"
      end
    end
  end

  desc "Remove admin role from a user by email"
  task :remove_admin, [ :email ] => :environment do |t, args|
    email = args[:email]

    if email.blank?
      puts "Error: Email is required."
      puts "Usage: rake admin:remove_admin[user@example.com]"
      exit 1
    end

    user = User.find_by(email: email)

    if user.nil?
      puts "Error: User with email '#{email}' not found."
      exit 1
    end

    # Get the shift_coordinator role
    admin_role = Role.find_by(name: "shift_coordinator")

    if admin_role.nil?
      puts "Error: 'shift_coordinator' role does not exist."
      exit 1
    end

    if !user.has_role?("shift_coordinator")
      puts "User '#{user.name}' does not have the admin role."
    else
      user.roles.delete(admin_role)
      puts "Successfully removed admin role from '#{user.name}' (#{user.email})."
    end
  end
end
